@@ -817,6 +817,10 @@ class IrrepsData:
     def factor_mul_to_last_axis(self, factor=None) -> "IrrepsData":
         if factor is None:
             factor = math.gcd(*(mul for mul, _ in self.irreps))
+
+        if not all(mul % factor == 0 for mul, _ in self.irreps):
+            raise ValueError(f"factor {factor} does not divide all multiplicities")
+
         irreps = Irreps([(mul // factor, ir) for mul, ir in self.irreps])
         list = [
             None if x is None else x.reshape(self.shape + (factor, mul, ir.dim))
@@ -964,26 +968,28 @@ class IrrepsData:
 
         return IrrepsData(irreps, self.contiguous, new_list)
 
-    def __add__(self, other):
+    def __add__(self, other: "IrrepsData") -> "IrrepsData":
         assert self.irreps == other.irreps
         list = [x if y is None else (y if x is None else x + y) for x, y in zip(self.list, other.list)]
         return IrrepsData(self.irreps, self.contiguous + other.contiguous, list)
 
-    def __sub__(self, other):
+    def __sub__(self, other: "IrrepsData") -> "IrrepsData":
         assert self.irreps == other.irreps
         list = [x if y is None else (-y if x is None else x - y) for x, y in zip(self.list, other.list)]
         return IrrepsData(self.irreps, self.contiguous - other.contiguous, list)
 
-    def __mul__(self, other):
-        list = [None if x is None else x * other for x in self.list]
-        return IrrepsData(self.irreps, self.contiguous * other, list)
+    def __mul__(self, other) -> "IrrepsData":
+        other = jnp.array(other)
+        list = [None if x is None else x * other[..., None, None] for x in self.list]
+        return IrrepsData(self.irreps, self.contiguous * other[..., None], list)
 
-    def __rmul__(self, other):
+    def __rmul__(self, other) -> "IrrepsData":
         return self * other
 
-    def __truediv__(self, other):
-        list = [None if x is None else x / other for x in self.list]
-        return IrrepsData(self.irreps, self.contiguous / other, list)
+    def __truediv__(self, other) -> "IrrepsData":
+        other = jnp.array(other)
+        list = [None if x is None else x / other[..., None, None] for x in self.list]
+        return IrrepsData(self.irreps, self.contiguous / other[..., None], list)
 
     @staticmethod
     def cat(args, axis="irreps"):
